package de.fitko.fitconnect.api.services;

import com.nimbusds.jose.jwk.JWK;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.*;

import java.io.File;
import java.util.List;
import java.util.UUID;

/**
 * A technical system that creates a submission via the FIT-Connect Submission API.
 * <p>
 * The Sender acts as a common interface wrapping all client functionality for authenticating <p>
 * and creating a valid {@link SubmissionSubmit} including encrypted {@link Data} and {@link Attachment}s
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/sending/overview">Sending Submissions</a>
 */
public interface Sender {

    /**
     * Validates the public key consisting of the following steps:
     * <p>
     * <ul>
     *     <li>checks if the JSON Web Key is suitable for the encryption</li>
     *     <li>checks if the public key is matching the certificate referenced in the JWK</li>
     *     <li>checks the certificate chain up to the root certificate</li>
     *     <li>checks against a certificate revocation list and/or an OSCP-endpoint with signed response</li>
     * </ul>
     * @param publicKey the public JWK
     *
     * @return {@link ValidationResult} that includes an error if the validation failed
     */
    ValidationResult validatePublicKey(final JWK publicKey);

    /**
     * Encrypts the submission data payload (json or xml) with JWE (JSON-Web-Encryption).
     *
     * @param publicKey the public encryption key for encryption of {@link Data}
     * @param data      the {@link Data} payload that should be encrypted
     * @return the JWE-encrypted {@link Data}
     *
     * @throws EncryptionException if the encryption fails due to an invalid key or corrupt data
     */
    Data encryptSubmissionData(final RSAKey publicKey, final Data data) throws EncryptionException;

    /**
     * Encrypts the submission attachment data with JWE (JSON-Web-Encryption).
     *
     * @param publicKey  the public encryption key for encryption of {@link Attachment}
     * @param attachment the {@link Attachment} that should be encrypted
     * @return the JWE-encrypted {@link Attachment}
     *
     * @throws EncryptionException
     */
    Attachment encryptAttachment(final RSAKey publicKey, final Attachment attachment) throws EncryptionException;

    /**
     * Authenticates the sender against the FIT-Connect API and retrieves an authentication token
     *
     * @param clientId     - a unique identifier within the FIT-Connect platform environment
     * @param clientSecret - a secret that is only known by the application and the OAuth server
     * @param scope        - OAuth scope(s) that determine if a submission is accepted by the client
     * @return {@link OAuthToken} that holds the access-token
     *
     * @throws {@link AuthenticationException} if an error occurred, including the original cause
     */
    OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope) throws AuthenticationException;

    /**
     * Creates and uploads a {@link Metadata} that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param data        {@link Data} object of the json or xml payload
     * @param attachments a list of {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s
     */
    Metadata createMetadata(final Data data, final List<Attachment> attachments) throws MetadataNotCreatedException;

    /**
     * Creates and uploads a {@link Metadata} that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param attachments a list of {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s
     *
     * @throws MetadataNotCreatedException if there was an error during the upload or on creation of the metadata object
     */
    Metadata createMetadata(final List<Attachment> attachments) throws MetadataNotCreatedException;

    /**
     * Creates and announces a new {@link SubmissionSubmit}.
     *
     * @param submission with a destinationId, a list of attachmentIds and a serviceType
     * @return the announced submission
     *
     * @throws SubmissionNotCreatedException if the announcement of the submission failed, e.g. due to a technical problem
     */
    SubmissionForPickup createSubmission(SubmissionSubmit submission) throws SubmissionNotCreatedException;

    /**
     * Posts the announced {@link SubmissionSubmit}
     *
     * @param submissionId      unique identifier of the announced submission
     * @param encryptedMetadata the encrypted {@link Metadata}
     * @return the submission that has been submitted
     *
     * @throws SubmissionNotSentException if the sending process failed
     */
    SubmissionForPickup sendSubmission(UUID submissionId, Metadata encryptedMetadata) throws SubmissionNotSentException;

    /**
     * Uploads encrypted {@link Attachment}s to for given submission id.
     *
     * @param submissionId unique identifier of the  submission
     * @param encryptedAttachments list of {@link Attachment}s
     *
     * @throws AttachmentUploadException
     */
    void uploadAttachments(UUID submissionId, List<Attachment> encryptedAttachments) throws AttachmentUploadException;

    /**
     * Creates a new {@link Attachment} from a given file.
     *
     * @param file file containing the {@link Attachment} payload.
     * @return an {@link Attachment} that contains the hashed file content as string
     *
     * @throws AttachmentCreationException if the file could not be loaded
     */
    Attachment createAttachment(File file) throws AttachmentCreationException;

    /**
     * Retrieves the public encryption key for a destination.
     *
     * @param destinationId unique identifier of the destination
     * @return the public key for encrypting {@link Metadata}, {@link Data} and {@link Attachment}s
     *
     * @throws KeyNotRetrievedException if a technical problem occurred or the destination id is not valid
     */
    RSAKey getEncryptionKeyForDestination(UUID destinationId) throws KeyNotRetrievedException;
}
